from mpi4py import MPI
from copy import deepcopy
import numpy as np

def run(m, log_periods_cnt, log_period_steps, log_callback, params_period_steps=None, relaxation_steps = None):
	if relaxation_steps is None:
		relaxation_steps = log_periods_cnt * log_period_steps

	if params_period_steps is None:
		params_period_steps = log_period_steps/30

	result = {}  # dict of temperatures to lists with log_callback() results  
	m.params_log_period = log_period_steps
	m.run(relaxation_steps)
	m.full_params_log = []
	m.params_log_period = params_period_steps
	for i in range(log_periods_cnt):
		m.run(log_period_steps)
		m.full_params_log += [deepcopy(m.params_log)]
		res_i = log_callback(m)
		result.setdefault(m.T_idx,list()).append(res_i)

	result = MPI.COMM_WORLD.gather(result,root=0)

	if MPI.COMM_WORLD.Get_rank() == 0:
		aggr = result[0]  # use root dict as initial for aggregated dict
		for dict_i in result[1:]:
			for T_idx, res_log in dict_i.iteritems():
				aggr.setdefault(T_idx, list()).extend(res_log)
				
		result = [res_log for T_idx, res_log in aggr.iteritems()]
		
		assert all([len(log_i) == log_periods_cnt for log_i in result]) # check for each temperature
	
	result = MPI.COMM_WORLD.bcast(result,root=0)

	full_params_log = np.concatenate(m.full_params_log)	# [time, param]
	full_params_log = MPI.COMM_WORLD.gather(full_params_log,root=0) # [rank, time, param]
	if MPI.COMM_WORLD.Get_rank() == 0:
		full_params_log = np.swapaxes(full_params_log, 0, 1)	# # [time, rank, param]
		
		# sort by the last element (T_idx)
		full_params_log = np.array([sorted(r, key=lambda v: v[-1]) for r in full_params_log]) # [time, T_idx, param]
		full_params_log = np.swapaxes(full_params_log, 0, 1)	# [T_idx, time, param]

	m.full_params_log = MPI.COMM_WORLD.bcast(full_params_log, root=0)


	return result
